/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvindexedstringlist.h
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#ifndef QVINDEXEDSTRINGLIST_H
#define QVINDEXEDSTRINGLIST_H

#include <QStringList>
#include <QMetaType>


/// @class QVIndexedStringList qvdta/qvindexedstringlist.h
/// @brief String List with a current string index.
///

#ifndef DOXYGEN_IGNORE_THIS
class QVIndexedStringList: public QStringList
	{
	private:
		int index;

	public:
		QVIndexedStringList() : QStringList(), index(-1) { }
		QVIndexedStringList( const QVIndexedStringList& l ) : QStringList(l), index(l.index) { }
		QVIndexedStringList( const QStringList& l ) : QStringList(l), index(l.isEmpty()? -1 : 0) { }
		QVIndexedStringList( const QString& i ) { append(i); index = 0; }

		//redefiniciones (los que devuelven listas y los que cambian ordenes)
		// quedan algunos métodos heredados que modifican la lista: erase, insert, removeAll, removeFirst, removeLast, Swap
		void sort();
		QVIndexedStringList filter(const QString &str, Qt::CaseSensitivity cs = Qt::CaseSensitive) const;
#ifndef QT_NO_REGEXP
		QVIndexedStringList filter(const QRegExp &rx) const;
#endif
		QVIndexedStringList operator+(const QVIndexedStringList &other) const;


		//nuevos (devuelve valor y modifican valos)
		int getIndex() const { return index; }
		QString getCurrent() const;
		bool setIndex(int i);
		bool incIndex();
		bool decIndex();
		void append(const QString &str);
	};

Q_DECLARE_METATYPE(QVIndexedStringList);
#endif
#endif

