/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

#include <qvgui/qv3dmodel.h>
#include <qvgui/qvglcanvas.h>

void QV3DModel::paint(QVGLCanvas &glWidget) const
	{
	glBegin(GL_LINES);
	for (int i=0; i< segmentList.size(); i++)
		{
		QV3DModelSegment s = segmentList.at(i);
		glColor3ub(s.r,s.g,s.b);
		glVertex3f(s.x0,s.y0,s.z0);
		glVertex3f(s.x1,s.y1,s.z1);
		}
	glEnd();

	for (int i=0; i< textList.size(); i++)
		{
		QV3DModelText text = textList.at(i);
		glColor3ub(text.r,text.g,text.b);
		glWidget.renderText(text.x, text.y, text.z, text.text);
		}

	if (imgList.size() > 0)
		{
		const QVImage<uChar> image= imgList.at(0);
		const uInt rows = image.getRows(), cols = image.getCols();		

		const double height = 1, width = (double)rows/(double)cols;

		const int coords[2][4][3] =	{
						{ { -width, -height, 0 }, { +width, -height, 0 }, { +width, +height, 0 }, { -width, +height, 0 } },
						{ { +width, -height, 0 }, { -width, -height, 0 }, { -width, +height, 0 }, { +width, +height, 0 } }
						};

		GLuint texture = glWidget.bindTexture(image, GL_TEXTURE_2D);

		glBindTexture(GL_TEXTURE_2D, texture);
		glBegin(GL_QUADS);
		for (int i = 0; i < 2; i++)
			for (int j = 0; j < 4; j++)
				{
				glTexCoord2d(j == 0 || j == 3, j == 0 || j == 1);
				glVertex3d(0.5 * coords[i][j][0], 0.5 * coords[i][j][1], 0.5 * coords[i][j][2]);
				}
		glEnd();

		glDeleteTextures(1,&texture);
		}
	}

QV3DModel QV3DModel::crossHair(const double size)
	{
	QV3DModel model;
	model.addSegment(-size,0,0,	+size,0,0,	255,255,0);
	model.addSegment(0,-size,0,	0,+size,0,	255,255,0);
	model.addSegment(0,0,-size,	0,0,+size,	255,255,0);
	return model;
	}

QV3DModel QV3DModel::cube(const double size)
	{
	QV3DModel model;
	model.addSegment(-size,+size,+size,	-size,-size,+size,	255,255,255);
	model.addSegment(-size,+size,+size,	-size,+size,-size,	255,255,255);
	model.addSegment(+size,-size,+size,	-size,-size,+size,	255,255,255);
	model.addSegment(+size,-size,+size,	+size,-size,-size,	255,255,255);
	model.addSegment(+size,+size,-size,	+size,-size,-size,	255,255,255);
	model.addSegment(+size,+size,-size,	-size,+size,-size,	255,255,255);
	model.addSegment(+size,-size,-size,	-size,-size,-size,	255,255,255);
	model.addSegment(-size,+size,-size,	-size,-size,-size,	255,255,255);
	model.addSegment(-size,-size,+size,	-size,-size,-size,	255,255,255);
	model.addSegment(+size,+size,+size,	-size,+size,+size,	255,255,255);
	model.addSegment(+size,+size,+size,	+size,-size,+size,	255,255,255);
	model.addSegment(+size,+size,+size,	+size,+size,-size,	255,255,255);

	return model;
	}

QV3DModel QV3DModel::image(QVImage<uChar> &img)
	{
	QV3DModel model;
	model.addImage(img);
	return model;
	}

QV3DModel QV3DModel::cuboid(const double sizex, const double sizey, const double sizez)
	{
	QV3DModel model;
	model.addSegment(-sizex,+sizey,+sizez,	-sizex,-sizey,+sizez,	255,255,255);
	model.addSegment(-sizex,+sizey,+sizez,	-sizex,+sizey,-sizez,	255,255,255);
	model.addSegment(+sizex,-sizey,+sizez,	-sizex,-sizey,+sizez,	255,255,255);
	model.addSegment(+sizex,-sizey,+sizez,	+sizex,-sizey,-sizez,	255,255,255);
	model.addSegment(+sizex,+sizey,-sizez,	+sizex,-sizey,-sizez,	255,255,255);
	model.addSegment(+sizex,+sizey,-sizez,	-sizex,+sizey,-sizez,	255,255,255);
	model.addSegment(+sizex,-sizey,-sizez,	-sizex,-sizey,-sizez,	255,255,255);
	model.addSegment(-sizex,+sizey,-sizez,	-sizex,-sizey,-sizez,	255,255,255);
	model.addSegment(-sizex,-sizey,+sizez,	-sizex,-sizey,-sizez,	255,255,255);
	model.addSegment(+sizex,+sizey,+sizez,	-sizex,+sizey,+sizez,	255,255,255);
	model.addSegment(+sizex,+sizey,+sizez,	+sizex,-sizey,+sizez,	255,255,255);
	model.addSegment(+sizex,+sizey,+sizez,	+sizex,+sizey,-sizez,	255,255,255);

	return model;
	}

QV3DModel QV3DModel::grid(const double stepX, const double stepY, const int cellsX, const int cellsY)
	{
	QV3DModel model;

	for (double i = -cellsX; i <= cellsX; i++)
		model.addSegment(stepX*i, stepY*cellsY, +0,	stepX*i, -stepY*cellsY , +0,	256,128,128);

	for (double j = -cellsY; j <= cellsY; j++)
		model.addSegment(stepX*cellsX, stepY*j, +0,	-stepX*cellsX, stepY*j , +0,	256,128,128);

	return model;
	}

QV3DModel QV3DModel::referenceCoordinates(const double size, const bool labelAxis)
	{
	QV3DModel model;
	model.addSegment(0,0,0,	+size,0,0,	255,0,0);
	model.addSegment(0,0,0,	0,+size,0,	0,255,0);
	model.addSegment(0,0,0,	0,0,+size,	0,0,255);

	if (labelAxis)
		{
		model.addText(1,0,0,"x", 255,255,255);
		model.addText(0,1,0,"y", 255,255,255);
		model.addText(0,0,1,"z", 255,255,255);
		}
	return model;
	}

