/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvfunctionminimizer.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <qvmath/qvfunctionminimizer.h>

double QVFunctionMinimizer::static_function(const double v, void *functionMinimizerPtr)
	{
	QVFunctionMinimizer *functionMinimizer = (QVFunctionMinimizer *) functionMinimizerPtr;
	functionMinimizer->actual = v;
	return functionMinimizer->function(functionMinimizer->actual);
	}

bool QVFunctionMinimizer::iterate(const int maxIterations, const double maxError)
	{
	// 1. Create GSL minimization structures.
	gsl_min_fminimizer *s =
		gsl_min_fminimizer_alloc (useGoldenSection? gsl_min_fminimizer_goldensection : gsl_min_fminimizer_brent);
	gsl_function F = { &static_function, this };
	gsl_min_fminimizer_set (s, &F, actual, minimum, maximum);

	// 2. Proceed to iterate.
	iterations = 0;
	int status = GSL_CONTINUE;
	while (status == GSL_CONTINUE && iterations++ <= maxIterations)
		if (status = gsl_min_fminimizer_iterate (s))
			break;
		else
			status = gsl_min_test_interval (gsl_min_fminimizer_x_lower (s), gsl_min_fminimizer_x_upper (s), 0.0, maxError);

	// 3. Release GSL minimization structures.
	gsl_min_fminimizer_free (s);

	// 4. Return optimization status.
	return (status == GSL_SUCCESS);
	}

